@@ -1,5 +1,5 @@
 /*
-** oidentd_masq.h - oidentd IP masquerading handler.
+** masq.h - oidentd IP masquerading handler.
 ** Copyright (c) 1998-2006 Ryan McCabe <ryan@numb.org>
 ** Copyright (c) 2018      Janik Rabe  <oidentd@janikrabe.com>
 **
@@ -37,7 +37,7 @@ int fwd_request(int sock,
 
 #endif
 
-bool masq(	int sock,
+int masq(	int sock,
 			in_port_t lport,
 			in_port_t fport,
 			struct sockaddr_storage *laddr,
